;=======================================================
;checks level info before modifying
;=======================================================
;This is called from LOADINITIALBITMAP code. It follows
;the main game "level load" call (which also makes reference
;to "Finished Loading!" string). This call is used for ALL
;level loads, thus including restarts and savegames. It should
;take place after CStartTriggers. If a breakpoint is placed here, it will typically
;halt loading with the progress bar at 100%.

;DO NOT RELY ON STACK - ALREADY NEAR LIMIT

;=======================================================
Modify_External:
;=======================================================

;----------------------------------------------
;Actions to perform at every level load (before all other function initializations)
;----------------------------------------------

cmp [NewGRFLoadDetected],FALSE
    jz NoENDLEVELLOADBFActions
	mov [LINST_enabled],FALSE
	mov [DisplayDino_INDEX],0
	call CreateMainCAnimalBuffer
NoENDLEVELLOADBFActions:

;----------------------------------------------
;Actions to perform at every level load ONLY if the level is not a saved game
;and level restart is important
;----------------------------------------------
	call IsLevelSavedGame
	     test eax,eax
		  jnz Saved_Game_No_Mods

	call SetGameDifficulty
	call ModifyCAnimalHP
	call SetRandomInstanceStartPositions

Saved_Game_No_Mods:

;----------------------------------------------
;Actions to perform only at new GRF loads (that may be savegames)
;----------------------------------------------
cmp [NewGRFLoadDetected],FALSE
    jz NoENDLEVELLOADGRFNEWAActions
	call ModifyCAnimalSleepWakeUp
NoENDLEVELLOADGRFNEWAActions:

;----------------------------------------------
;Actions to perform at every level load
;----------------------------------------------

	call SaveCAnimalInitialData		;must be after instance randomizer call
	call DoAT10JUMPMainEmulInit
	call DoAnimalProjectileInit

	call DoNightModeTeleportsInit

   ;PRINT CANIMAL COUNT STRING FOR CANIMAL CHEAT
	call GetCAnimalInstanceCount
invoke wsprintf,canimal_title,canimal_title_base,eax
	add esp,12
	mov [canimal_title_size],eax

   ;RESET PLAYER'S AITYPE AND MANAGE AT113
	call FLY_CLOAK_Cheats_Reset
	mov [AT113_PLAYER_AITYPE_NON5_SET],FALSE	;set to true if originally enabled during savegame creation
	cmp [AT113_SavegameAITypeUsed_BOOL],TRUE
	    jnz AT113_NoSavegameAITypeValue
	    mov [AT113_PLAYER_AITYPE_NON5_SET],TRUE
	    push [AT113_SavegameAITypeValue]
	    call CHANGEPLAYERTYPE_PROC
	AT113_NoSavegameAITypeValue:

   ;SET IN-GAME KEYS AND MOUSE MOVEMENT AND MANAGE AT112
	call EnableInGameKeys
	mov [DisableInGameMouseMvt],FALSE
	mov ecx,(AT112_InputTogglerID-8)		;(we know how AT112 works, so a little hack can't hurt...)
	call AT112MAIN

   ;CPARTICLES CODE
	call CParticlesEndLevelLoad	;MUST BE BEFORE "and [Restart_Exit_Level_BOOL],0"

   ;SKY AND FOG MODIFICATIONS
	call SetSkyFogColorNearFarInit	;(used for when game does not summon CStartTriggers)

   ;SET FIRST FRAME AS COUNTER START (TIME CHEAT)
	mov [CounterFirstRun],1

   ;ENSURE SOME CHEAT DISABLED
	mov [ATXDRV_ENABLED],FALSE
	xor al,al
	or al,[FD_enabled]
	or al,[THIRDP_enabled]
	mov [FD_enabled],FALSE
	mov [THIRDP_enabled],FALSE
	mov [THIRDP_keyaimtoggle_ON],FALSE
	mov [OBJN_instance],NULL
	mov [UseObjCheat],FALSE
	test al,al
	     jz NoEndLevelLoadCCameraObjSourceReset
	     call ResetCCameraPlayerSources
	NoEndLevelLoadCCameraObjSourceReset:

;----------------------------------------------
;Actions to perform after all others at every level start
;----------------------------------------------

	call InitPlayerPathTraces

	call AdjustCameraFOVWidescreen
	mov [THIRDP_aimdot_lastobj],0

	mov [Restart_Exit_Level_BOOL],0
	mov [IsInGame_BOOL],1
	call StartInGameMenuTimer
	mov [IsInGame_BOOL],0
	mov [NotInGame_TOTALMENUTIME],0
	invoke GetTickCountTRUE
	mov [LevelStart_TICK],eax

	mov [QUICKSAVE_tick],0
	mov [QUICKSAVE_savedone_BOOL],0
	mov [TempMessageTime],0

	call FIXOBJECTSPEED_INIT		;must be after DoAnimalProjectileInit call


ret	;(this marks the end of ATX features used during level load)

;=======================================================
FLY_CLOAK_Cheats_Reset:   ;eax=0 if buffers not OK
;=======================================================
	mov [FLY_MODE_BOOL],0
	mov eax,[PLAYER_OBJ_INVUL_ADDR]
	mov edx,[eax]		;edx = main "Player" class object
	and BYTE [edx+0DDh],0FEh	;disable INVUL
	push 5
	call CHANGEPLAYERTYPE_PROC
ret

;================================================================
ModifyCAnimalHP:
;================================================================
;CAnimal instance (all) modification code at level start (for HitPoints)
;note: the game does not move the instances' locations in memory (during the period
;between the instancing and this code) so the pointers should
;always be valid
push esi
push edi

cmp [UseCAnimalHitPoints],0
    jz NoModCAnimalInst
    mov esi,[CAnimalBuffer]
    test esi,esi
	 jz NoModCAnimalInst

	CAnimalModLoop:
	mov edi,[esi+canimal_info.Instance]
;modify instance's health values (updated after each level restart)
;add to HitPoints
	fld DWORD [edi+50h]
	fadd [UseCAnimalHitPoints]
	fstp DWORD [edi+50h]
;add to MaxHitPoints
	fld DWORD [edi+54h]
	fadd [UseCAnimalHitPoints]
	fstp DWORD [edi+54h]
	add esi,sizeof.canimal_info
	cmp esi,[CAnimalBufferEnd]
	    jb CAnimalModLoop

NoModCAnimalInst:
pop edi
pop esi
ret

;===============================================================
ModifyCAnimalSleepWakeUp:     ;adds Sleep and WakeUp values to all dinos
;===============================================================
push edi
	mov edi,[CAnimalBuffer]
	test edi,edi
	     jz NoModifyCAnimal_ONCEONLY_ALLINSTANCES

ModifyCAnimalAllOnceOnly_LOOP:
	mov eax,[edi+canimal_info.Instance]
;add to instance's Sleep/WakeUp values (not updated after level restart)
	mov edx,[eax+0DCh]	   ;CBrain object
;add to WakeUp
	fld DWORD [edx+340h]
	fadd [UseSleepWake]
	fstp DWORD [edx+340h]
;add to Sleep
	fld DWORD [edx+344h]
	fadd [UseSleepWake]
	fstp DWORD [edx+344h]
	add edi,sizeof.canimal_info
	cmp edi,[CAnimalBufferEnd]
	    jb ModifyCAnimalAllOnceOnly_LOOP

NoModifyCAnimal_ONCEONLY_ALLINSTANCES:
pop edi
ret

;===============================================================
SaveCAnimalInitialData:
;===============================================================
push edi
push esi
	mov ebx,[CAnimalBuffer]
	test ebx,ebx
	     jz NoSaveCAnimalInitialData
	SaveCAnimalInitialDataLOOP:
	mov eax,[ebx+canimal_info.Instance]
	lea edi,[ebx+canimal_info.InitialData]
	lea esi,[eax+30h]
	mov ecx,7
	rep movsd
	add ebx,sizeof.canimal_info
	cmp ebx,[CAnimalBufferEnd]
	    jb SaveCAnimalInitialDataLOOP

NoSaveCAnimalInitialData:
pop esi
pop edi
ret


;===============================================================
CreateMainCAnimalBuffer:	;only creates buffer if one or more dino instances
;===============================================================
push edi
push esi
push ebx

     LOCALRELEASE [CAnimalBuffer]
     xor eax,eax
     mov [CAnimalBufferSize],eax
     mov [CAnimalBufferEnd],eax
     mov [CAnimal_Intance_Count],eax

;get number of CAnimal Instances
     mov ecx,[CANIMALCCREATRIGBUFADDR]	  ;this chain object buffer contains CAnimal and CCreatureTrigger objects (and others?)
     cmp DWORD [ecx+4],0
	 jz NoCreateMainCAnimalBuffer

     mov ecx,[ecx]			  ;ecx = first chain object (dummy)
     test ecx,ecx
	  jz NoCreateMainCAnimalBuffer
     mov esi,[ecx]
     cmp esi,ecx			  ;make sure the dummy object isn't the
	 jz NoCreateMainCAnimalBuffer	    ;the last one, or there is an error in the list

     xor ebx,ebx
     mov edx,[CANIMAL_VTABLE]

     CreateMainCAnimalBufferCopyInstanceLOOP:
     mov eax,[esi+8]			  ;CAnimal instance
     mov esi,[esi]			  ;next chain object
     add edi,sizeof.canimal_info	  ;next buffer member
     cmp [eax],edx			  ;is object of CAnimal type?
	 jnz NoIncreaseCAnimalCount
     inc ebx
     NoIncreaseCAnimalCount:
     cmp esi,ecx
	 jnz CreateMainCAnimalBufferCopyInstanceLOOP	;have we wrapped around to dummy instance?

     test ebx,ebx
     mov [CAnimal_Intance_Count],ebx
	  jz NoCreateMainCAnimalBuffer

;allocate space for one canimal_info struc for each CAnimal instance
     mov eax,sizeof.canimal_info
     mul ebx
     mov [CAnimalBufferSize],eax
     invoke LocalAlloc,LPTR,eax
     test eax,eax
     mov edi,eax
	 jz NoCreateMainCAnimalBuffer

     mov [CAnimalBuffer],eax

;copy instances from game to CAnimalBuffer

     mov ecx,[CANIMALCCREATRIGBUFADDR]	  ;this chain object buffer contains CAnimal and CCreatureTrigger objects (and others?)   ;first instance in CAnimal instance "chain" (first is dummy)
     mov ecx,[ecx]

;"chain" object format:
;DWORD [+00h] = pointer to next "chain object" (points to first dummy object if last)
;DWORD [+04h] = pointer to previous "chain object" (points to last object if first)
;DWORD [+08h] = CAnimal instance pointer

     mov esi,[ecx]
     cmp esi,ecx			  ;make sure the dummy object isn't the
	 jz CreateMainCAnimalBuffer	  ;the last one, or there is an error in the list

;ecx = first (dummy) chain object
     xor ebx,ebx
     mov edx,[CANIMAL_VTABLE]

     CreateMainCAnimalBufferCopyInstanceLOOP_2:
     mov eax,[esi+8]			  ;CAnimal instance
     mov esi,[esi]			  ;next chain object
     cmp [eax],edx
	 jnz NoCopyInstanceNotCAnimal
	 mov [edi+canimal_info.Instance],eax  ;store instance
	 mov [edi+canimal_info.Count],ebx     ;store instance number (relative to buffer start)
	 inc ebx
	 add edi,sizeof.canimal_info	      ;next buffer member
     NoCopyInstanceNotCAnimal:
     cmp esi,ecx
	 jnz CreateMainCAnimalBufferCopyInstanceLOOP_2	  ;have we wrapped around to dummy instance?

     mov eax,[CAnimalBuffer]
     add eax,[CAnimalBufferSize]
     mov [CAnimalBufferEnd],eax

pop ebx
pop esi
pop edi
ret


NoCreateMainCAnimalBuffer:
     LOCALRELEASE [CAnimalBuffer]
     mov [CAnimalBufferSize],0
pop ebx
pop esi
pop edi
ret
;===============================================================

