;============================================================
;FIX OBJECT SPEED AT110 REAL-TIME IMPLEMENTATION
;============================================================
FIXOBJECTSPEED_RT:  ;always called if main buffer is non-null
push ebp

	invoke GetTickCount
	sub eax,[FIXS_lasttickglobal]
	mov ebp,eax			 ;tick difference between frames

	mov esi,[FIXS_mainbuffer]
	mov ebx,[FIXS_mainbuffer_sizediv2]
	xor eax,eax

	FIXOBJECTSPEED_RT_MAINLOOP:
	cmp [esi],eax
	mov edi,[esi+ebx]
	    jz FIXOBJECTSPEED_RT_MAINLOOP_NOINSTANCEID

	    mov eax,[edi+at110proj_info.TickLeft]
	    cmp eax,-1
		jz FIXOBJECTSPEED_RT_MAINLOOP_InstanceTimeLeft
	    sub eax,ebp
		ja FIXOBJECTSPEED_RT_MAINLOOP_InstanceTimeLeft
		mov DWORD [esi],0		   ;erase instance name ID
		invoke LocalFree,[esi+ebx]	   ;free structure
		jmp FIXOBJECTSPEED_RT_MAINLOOP_NOINSTANCEID
	    FIXOBJECTSPEED_RT_MAINLOOP_InstanceTimeLeft:
	    mov [edi+at110proj_info.TickLeft],eax	 ;update remaining tick time

	 ;set instance velocity via ActionType 10
	    mov ecx,AT10_SIM_OBJECT_VELONLY
	    mov eax,[edi+at110proj_info.Instance]
	    mov edx,[edi+at110proj_info.Velocity.X]
	    mov [ecx+CSetPhysicsAction.TargetInstance],eax
	    mov [ecx+CSetPhysicsAction.Velocity.X],edx
	    mov eax,[edi+at110proj_info.Velocity.Y]
	    mov edx,[edi+at110proj_info.Velocity.Z]
	    mov [ecx+CSetPhysicsAction.Velocity.Y],eax
	    mov [ecx+CSetPhysicsAction.Velocity.Z],edx

	    mov edx,[ecx]
	    mov [ecx+CSetPhysicsAction.RefCount],0
	    mov [ecx+CSetPhysicsAction.Facing.Y],1.000000
	    call [edx+ICAction.MainProc]	       ;finally, set speed at every frame

	FIXOBJECTSPEED_RT_MAINLOOP_NOINSTANCEID:
	add esi,4
	xor eax,eax
	cmp esi,[FIXS_mainbuffer_halfend]
	    jb FIXOBJECTSPEED_RT_MAINLOOP

	add [FIXS_lasttickglobal],ebp

pop ebp
ret


;============================================================
FIXOBJECTSPEED_INIT:		;buffer released in PRELEVELLOAD.inc
;============================================================
push ebp
push ebx
push esi
push edi

	 mov eax,[FIXS_numberofAT110instances]
	 add eax,[FIXS_numberofSPTprojectile]
	 shl eax,2
	 mov [esp-4],eax
	 shl eax,1
	 test eax,eax
	      jz FIXOBJECTSPEED_INIT_FORCEEXIT
	 sub esp,4
	 invoke LocalAlloc,LPTR,eax
	 test eax,eax
	 pop ecx
	     jz FIXOBJECTSPEED_INIT_FORCEEXIT
	 mov [FIXS_mainbuffer],eax		 ;possible previous buffers released in PRELEVELLOAD.inc
	 mov ebx,eax
	 mov [FIXS_mainbuffer_sizediv2],ecx
	 add ecx,eax
	 mov [FIXS_mainbuffer_halfend],ecx

   ;if we loaded a savegame containing a saved buffer, initialize its contents and release
   ;format is (following 8 first bytes): DWORD (instance name ID0, structure, DWORD (instance name ID0, structure, etc.
	 mov esi,[FIXS_savedatatempbuffer]
	 mov edi,[FIXS_savedatatempbuffer_size]
	 test esi,esi
	      jz NoFIXSinitsavedatatempbuffer
	 cmp edi,(5*4+8+4)
	     jb NoFIXSinitsavedatatempbuffer
	 mov ebp,[esi+04]			;struc size
	 cmp edi,ebp				;at least this size required (one instance name + struc)
	     jb NoFIXSinitsavedatatempbuffer
	 cmp DWORD [esi],152			;currently, this is the only existing AT110 version
	     jnz NoFIXSinitsavedatatempbuffer

	 add edi,esi
	 add esi,8

	 FIXSinitsavedatatempbuffer_LOOP:
	 mov edx,[LocationADDR]
	 push DWORD [esi]
	 mov ecx,[edx]
	 call [GETOBJFROMSTRINGIDPROC]	     ;this fetched the string ID's associated instance, if existing
	 test eax,eax
	      jz FIXSinitsavedatatempbuffer_LOOP_NoInstance   ;is the instance still valid in this level? (should always be)

	     push eax			;save instance itself
	     invoke LocalAlloc,LPTR,sizeof.at110proj_info	  ;allocate space for structure (all members NULL by default)
	     test eax,eax
	     pop edx
		 jz END_FIXSinitsavedatatempbuffer_LOOP
		 push eax
		 push esi
		 push edi
		      mov edi,eax
		      lea esi,[esi+4]
		      mov ecx,ebp
		      rep movsb 		  ;copy structure to new buffer
		 pop edi
		 pop esi
		 pop eax
		 mov ecx,[FIXS_mainbuffer_sizediv2]
		 mov [eax+at110proj_info.Instance],edx		       ;set instance in struct
		 mov [ebx+ecx],eax				       ;store structure in final buffer
		 mov ecx,[esi]		    ;get instance ID again
		 mov [ebx],ecx		    ;store instance ID as valid
	 FIXSinitsavedatatempbuffer_LOOP_NoInstance:

	 add ebx,4
	 cmp ebx,[FIXS_mainbuffer_halfend]			       ;are we out of space? (should not happen)
	     jnb END_FIXSinitsavedatatempbuffer_LOOP
	 lea esi,[esi+ebp+4]
	 cmp esi,edi
	     jb FIXSinitsavedatatempbuffer_LOOP
     END_FIXSinitsavedatatempbuffer_LOOP:

	 NoFIXSinitsavedatatempbuffer:
	 LOCALRELEASE [FIXS_savedatatempbuffer]
	 mov [FIXS_savedatatempbuffer_size],0

	 invoke GetTickCount
	 mov [FIXS_lasttickglobal],eax

pop edi
pop esi
pop ebx
pop ebp
ret


FIXOBJECTSPEED_INIT_FORCEEXIT:
	 call ReleaseFIXOBJECTSPEEDmainbuffer

	 LOCALRELEASE [FIXS_savedatatempbuffer]
	 mov [FIXS_savedatatempbuffer_size],0

pop edi
pop esi
pop ebx
pop ebp
ret


;============================================================
ReleaseFIXOBJECTSPEEDmainbuffer:
;============================================================
push esi
push edi
push ebx
	mov esi,[FIXS_mainbuffer]
	mov edi,[FIXS_mainbuffer_sizediv2]
	test esi,esi
	     jz NoReleaseFIXOBJECTSPEEDmainbuffer
	mov ebx,[FIXS_mainbuffer_halfend]

	ReleaseFIXOBJECTSPEEDmainbuffer_LOOP:
	cmp DWORD [esi],0
	    jz NoLocalReleaseSubStrucFIXOBJECTSPEED
	    invoke LocalFree,[esi+edi]
	NoLocalReleaseSubStrucFIXOBJECTSPEED:
	add esi,4
	cmp esi,ebx
	    jb ReleaseFIXOBJECTSPEEDmainbuffer_LOOP

	LOCALRELEASE [FIXS_mainbuffer]

NoReleaseFIXOBJECTSPEEDmainbuffer:
	xor eax,eax
	mov [FIXS_mainbuffer_sizediv2],eax
	mov [FIXS_mainbuffer_halfend],eax

pop ebx
pop edi
pop esi
ret
